@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2008 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2009 Alexandru Munteanu - io_fx@yahoo.fr
  *
  * http://mp3splt.sourceforge.net
  *
@@ -133,7 +133,7 @@ FILE *splt_ogg_open_file_read(splt_state *state,
   else
   {
     //we open the file
-    file_input = fopen(filename, "rb");
+    file_input = splt_u_fopen(filename, "rb");
     if (file_input == NULL)
     {
       splt_t_set_strerror_msg(state);
@@ -335,7 +335,6 @@ static int splt_ogg_write_pages_to_file(splt_state *state,
   return 0;
 
 write_error:
-  splt_t_set_strerror_msg(state);
   splt_t_set_error_data(state, output_fname);
   *error = SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
   return -1;
@@ -378,7 +377,7 @@ static void splt_ogg_v_free(splt_ogg_state *oggstate)
         splt_ogg_free_packet(oggstate->headers[i]);
       }
       free(oggstate->headers);
-      oggstate->headers;
+      oggstate->headers = NULL;
     }
     vorbis_comment_clear(&oggstate->vc);
     if(oggstate->vb)
@@ -643,7 +642,8 @@ void splt_ogg_put_tags(splt_state *state, int *error)
     //if we put current tags (cddb,cue,...)
     if (splt_t_get_int_option(state, SPLT_OPT_TAGS) == SPLT_CURRENT_TAGS)
     {
-      int current_split = splt_t_get_current_split(state);
+      int current_split = splt_t_get_current_split_file_number(state) - 1;
+      int old_current_split = current_split;
 
       //if we set all the tags like the x one
       int remaining_tags_like_x = splt_t_get_int_option(state,SPLT_OPT_ALL_REMAINING_TAGS_LIKE_X); 
@@ -659,31 +659,28 @@ void splt_ogg_put_tags(splt_state *state, int *error)
         char *title = NULL;
         char *artist = NULL;
 
-        title = splt_t_get_tags_char_field(state,current_split,
-              SPLT_TAGS_TITLE);
-        artist = splt_t_get_tags_char_field(state,current_split,
-              SPLT_TAGS_ARTIST);
+        title = splt_t_get_tags_char_field(state,current_split, SPLT_TAGS_TITLE);
+        artist = splt_t_get_tags_char_field(state,current_split, SPLT_TAGS_ARTIST);
 
         //only if we have the artist or the title
         if (((artist != NULL) && (artist[0] != '\0'))
             || ((title != NULL) && (title[0] != '\0')))
         {
+          splt_t_set_auto_increment_tracknumber_tag(state, old_current_split, current_split);
+
           int tags_number = 0;
-          splt_tags *tags = 
-            splt_t_get_tags(state, &tags_number);
+          splt_tags *tags = splt_t_get_tags(state, &tags_number);
 
           if (splt_t_tags_exists(state, current_split))
           {
             char *track_string = NULL;
             if (tags[current_split].track > 0)
             {
-              track_string = 
-                splt_ogg_trackstring(tags[current_split].track);
+              track_string = splt_ogg_trackstring(tags[current_split].track);
             }
             else
             {
-              track_string = 
-                splt_ogg_trackstring(current_split+1);
+              track_string = splt_ogg_trackstring(current_split+1);
             }
 
             if (track_string)
@@ -1101,7 +1098,7 @@ static int splt_ogg_find_begin_cutpoint(splt_state *state, splt_ogg_state *oggst
  */
 static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *stream,
     FILE *in, FILE *f, ogg_int64_t cutpoint, short adjust, float threshold,
-    int *error, const char *output_fname)
+    int *error, const char *output_fname, int save_end_point)
 {
   splt_t_put_progress_text(state,SPLT_PROGRESS_CREATE);
 
@@ -1117,8 +1114,6 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
   ogg_int64_t page_granpos = 0, current_granpos = 0, prev_granpos = 0;
   ogg_int64_t packetnum=0; /* Should this start from 0 or 3 ? */
 
-  char *filename = splt_t_get_filename_to_split(state);
-
   if(oggstate->packets[0] && oggstate->packets[1])
   { // Check if we have the 2 packet, begin can be 0!
     packet.bytes = oggstate->packets[0]->length;
@@ -1356,13 +1351,22 @@ static int splt_ogg_find_end_cutpoint(splt_state *state, ogg_stream_state *strea
   }
 
   oggstate->initialgranpos = prev_granpos - cutpoint;
-  oggstate->end = 1;
   oggstate->cutpoint_begin += cutpoint;
 
+  if (save_end_point)
+  {
+    oggstate->end = 1;
+  }
+  else
+  {
+    oggstate->end = 0;
+    //if we don't save the end point, go at the start of the file
+    rewind(oggstate->in);
+  }
+
   return 0;
 
 write_error:
-  splt_t_set_strerror_msg(state);
   splt_t_set_error_data(state, output_fname);
   *error = SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
   return -1;
@@ -1371,10 +1375,8 @@ write_error:
 //splits ogg
 void splt_ogg_split(const char *output_fname, splt_state *state, double
     sec_begin, double sec_end, short seekable, 
-    short adjust, float threshold, int *error)
+    short adjust, float threshold, int *error, int save_end_point)
 {
-  //we do the next split
-  splt_t_current_split_next(state);
   splt_ogg_state *oggstate = state->codec;
 
   ogg_stream_state stream_out;
@@ -1432,7 +1434,7 @@ void splt_ogg_split(const char *output_fname, splt_state *state, double
   }
   else
   {
-    if (!(oggstate->out = fopen(output_fname, "wb")))
+    if (!(oggstate->out = splt_u_fopen(output_fname, "wb")))
     {
       splt_t_set_strerror_msg(state);
       splt_t_set_error_data(state, output_fname);
@@ -1470,8 +1472,8 @@ void splt_ogg_split(const char *output_fname, splt_state *state, double
   }
 
   //find end cutpoint and get error
-  int result = splt_ogg_find_end_cutpoint(state, &stream_out, oggstate->in, 
-      oggstate->out, cutpoint, adjust, threshold, error, output_fname);
+  splt_ogg_find_end_cutpoint(state, &stream_out, oggstate->in, 
+      oggstate->out, cutpoint, adjust, threshold, error, output_fname, save_end_point);
 
 end:
   ogg_stream_clear(&stream_out);
@@ -1733,13 +1735,17 @@ int splt_ogg_scan_silence(splt_state *state, short seconds,
       //progress bar
       if (splt_t_get_int_option(state,SPLT_OPT_SPLIT_MODE) == SPLT_OPTION_SILENCE_MODE)
       {
-        float level = splt_u_convert2dB(oggstate->temp_level);
-        if (state->split.get_silence_level)
+        //if (count++ % 10 == 0)
         {
-          state->split.get_silence_level(level, state->split.silence_level_client_data);
+          float level = splt_u_convert2dB(oggstate->temp_level);
+          if (state->split.get_silence_level)
+          {
+            state->split.get_silence_level( (unsigned long) (((double) pos / oggstate->vi->rate) * 100.0),
+                level, state->split.silence_level_client_data);
+          }
+          state->split.p_bar->silence_db_level = level;
+          state->split.p_bar->silence_found_tracks = found;
         }
-        state->split.p_bar->silence_db_level = level;
-        state->split.p_bar->silence_found_tracks = found;
 
         //if we have cancelled the split
         if (splt_t_split_is_canceled(state))
@@ -1821,7 +1827,7 @@ int splt_pl_check_plugin_is_for_file(splt_state *state, int *error)
   char *filename = splt_t_get_filename_to_split(state);
 
   //o- means stdin ogg format
-  if (strcmp(filename,"o-") == 0)
+  if ((filename != NULL) && (strcmp(filename,"o-")) == 0)
   {
     return SPLT_TRUE;
   }
@@ -1831,7 +1837,7 @@ int splt_pl_check_plugin_is_for_file(splt_state *state, int *error)
 
   FILE *file_input = NULL;
 
-  if ((file_input = fopen(filename, "rb")) == NULL)
+  if ((file_input = splt_u_fopen(filename, "rb")) == NULL)
   {
     splt_t_set_strerror_msg(state);
     splt_t_set_error_data(state,filename);
@@ -1882,15 +1888,12 @@ void splt_pl_init(splt_state *state, int *error)
 
 void splt_pl_end(splt_state *state, int *error)
 {
-  splt_ogg_state *oggstate = state->codec;
   splt_ogg_state_free(state);
 }
 
 void splt_pl_split(splt_state *state, const char *final_fname,
-    double begin_point, double end_point, int *error) 
+    double begin_point, double end_point, int *error, int save_end_point) 
 {
-  splt_ogg_state *oggstate = state->codec;
-
   splt_ogg_put_tags(state, error);
 
   if (*error >= 0)
@@ -1900,7 +1903,7 @@ void splt_pl_split(splt_state *state, const char *final_fname,
         begin_point, end_point,
         !state->options.option_input_not_seekable,
         state->options.parameter_gap,
-        state->options.parameter_threshold, error);
+        state->options.parameter_threshold, error, save_end_point);
   }
 }
 
