@@ -3,7 +3,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2008 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2009 Alexandru Munteanu - io_fx@yahoo.fr
  *
  *********************************************************/
 
@@ -51,6 +51,7 @@ static void splt_tag_put_filenames_from_tags(splt_state *state,
     int tracks, int *error)
 {
   int i = 0;
+
   char *artist0 = NULL;
   artist0 = strdup(splt_t_get_tags_char_field(state, 0, SPLT_TAGS_ARTIST));
   if (artist0 == NULL)
@@ -180,6 +181,8 @@ static void splt_tag_put_filenames_from_tags(splt_state *state,
       }
     }
 
+    splt_t_set_oformat_digits_tracks(state, tracks);
+
     if (err_format >= 0)
     {
       //we set the current split to 0
@@ -235,77 +238,82 @@ static int splt_cue_set_value(splt_state *state, char *in,
 
   char *ptr_b = in, *ptr_e = NULL;
 
-  //-find begin
-  //while we don't have spaces any more
-  while (*ptr_b == ' ')
-  {
-    ptr_b++;
-  }
-  //if we have a '"', then skip quote
-  if (*ptr_b == '"')
-  {
-    ptr_b++;
-  }
-  
-  //-find and and put '\0'
-  //go to the end of line
-  ptr_e = strchr(ptr_b+1,'\n');
-  //move back ignoring spaces
-  while (*ptr_e == ' ')
-  {
-    ptr_e--;
-  }
-  //if we have a '"', then skip quote
-  if (*ptr_e == '"')
-  {
-    ptr_e--;
-  }
-  *ptr_e = '\0';
+	if (in)
+	{
+		//-find begin
+		//while we don't have spaces any more
+		while (*ptr_b == ' ')
+		{
+			ptr_b++;
+		}
+		//if we have a '"', then skip quote
+		if (*ptr_b == '"')
+		{
+			ptr_b++;
+		}
 
-  char *out = NULL;
-  if ((out = malloc(strlen(ptr_b)+1)) == NULL)
-  {
-    error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-  }
-  else
-  {
-    strncpy(out, ptr_b, (strlen(ptr_b)+1));
-    splt_u_cleanstring(state, out, &error);
-    if (error >= 0)
-    {
-      int tags_err = SPLT_OK;
+		//-find and and put '\0'
+		//go to the end of line
+		ptr_e = strchr(ptr_b+1,'\n');
+		if (ptr_e)
+		{
+			//move back ignoring spaces
+			while (*ptr_e == ' ')
+			{
+				ptr_e--;
+			}
+			//if we have a '"', then skip quote
+			if (*(ptr_e-1) == '"')
+			{
+				ptr_e--;
+			}
+			*ptr_e = '\0';
+		}
 
-      //put Artist + Album info to client
-      char *client_infos = malloc(sizeof(char) * (strlen(out)+30));
-      if (client_infos == NULL)
-      {
-        error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
-      }
-      else
-      {
-        if (tag_field == SPLT_TAGS_ARTIST)
-        {
-          snprintf(client_infos,strlen(out)+30,"\n  Artist: %s\n", out);
-          splt_t_put_message_to_client(state, client_infos);
-        }
-        else if (tag_field == SPLT_TAGS_ALBUM)
-        {
-          snprintf(client_infos,strlen(out)+30,"  Album: %s\n", out);
-          splt_t_put_message_to_client(state, client_infos);
-        }
-        free(client_infos);
-        client_infos = NULL;
+		char *out = NULL;
+		if ((out = malloc(strlen(ptr_b)+1)) == NULL)
+		{
+			error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+		}
+		else
+		{
+			strncpy(out, ptr_b, (strlen(ptr_b)+1));
+			int tags_err = SPLT_OK;
 
-        tags_err = splt_t_set_tags_char_field(state, index, tag_field, out);
-        if (tags_err != SPLT_OK)
-        {
-          error = tags_err;
-        }
-      }
-    }
-    free(out);
-    out = NULL;
-  }
+			//put Artist + Album info to client
+			char *client_infos = malloc(sizeof(char) * (strlen(out)+30));
+			if (client_infos == NULL)
+			{
+				error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+			}
+			else
+			{
+				if (tag_field == SPLT_TAGS_ARTIST)
+				{
+					snprintf(client_infos,strlen(out)+30,"\n  Artist: %s\n", out);
+					splt_t_put_message_to_client(state, client_infos);
+				}
+				else if (tag_field == SPLT_TAGS_ALBUM)
+				{
+					snprintf(client_infos,strlen(out)+30,"  Album: %s\n", out);
+					splt_t_put_message_to_client(state, client_infos);
+				}
+				free(client_infos);
+				client_infos = NULL;
+
+				tags_err = splt_t_set_tags_char_field(state, index, tag_field, out);
+				if (tags_err != SPLT_OK)
+				{
+					error = tags_err;
+				}
+			}
+			if (out)
+			{
+				free(out);
+				out = NULL;
+			}
+		}
+	}
 
   return error;
 }
@@ -364,7 +372,7 @@ int splt_cue_put_splitpoints(const char *file, splt_state *state, int *error)
   }
   
   //if we cannot open the file
-  if (!(file_input=fopen(file, "r")))
+  if (!(file_input=splt_u_fopen(file, "r")))
   {
     splt_t_set_strerror_msg(state);
     splt_t_set_error_data(state,file);
@@ -504,9 +512,6 @@ int splt_cue_put_splitpoints(const char *file, splt_state *state, int *error)
             {
               //we replace : with . for the sscanf
               ptr[dot-ptr] = ptr[dot-ptr+3] = '.';
-              //we clean the string for unwanted characters
-              splt_u_cleanstring(state, ptr, error);
-              if (*error < 0) { goto function_end; }
 
               //we convert to hundreths of seconds and put splitpoints
               if (tracks>0)
@@ -521,7 +526,7 @@ int splt_cue_put_splitpoints(const char *file, splt_state *state, int *error)
 
                 //we append the splitpoint
                 append_error =
-                  splt_t_append_splitpoint(state, hundr_seconds, NULL);
+                  splt_t_append_splitpoint(state, hundr_seconds, NULL, SPLT_SPLITPOINT);
                 if (append_error != SPLT_OK)
                 {
                   *error = append_error;
@@ -539,7 +544,7 @@ int splt_cue_put_splitpoints(const char *file, splt_state *state, int *error)
       }
       //we append the last splitpoint
       append_error = splt_t_append_splitpoint(state, LONG_MAX,
-          "description here");
+          "description here", SPLT_SPLITPOINT);
     }
     else
     {
@@ -637,7 +642,7 @@ int splt_cddb_put_splitpoints (const char *file, splt_state *state, int *error)
   char *perfor = NULL;
 
   //we open the file
-  if (!(file_input=fopen(file, "r")))
+  if (!(file_input=splt_u_fopen(file, "r")))
   {
     splt_t_set_strerror_msg(state);
     splt_t_set_error_data(state,file);
@@ -696,7 +701,7 @@ int splt_cddb_put_splitpoints (const char *file, splt_state *state, int *error)
           //in cddb_offset*100
           //we convert them lower to seconds
           append_error =
-            splt_t_append_splitpoint(state, temp * 100, NULL);
+            splt_t_append_splitpoint(state, temp * 100, NULL, SPLT_SPLITPOINT);
   
           if (append_error != SPLT_OK)
           {
@@ -753,7 +758,7 @@ int splt_cddb_put_splitpoints (const char *file, splt_state *state, int *error)
         temp2 = atof(number);
 
         //we append the splitpoint (in seconds*100)
-        append_error = splt_t_append_splitpoint(state, temp2 * 100, NULL);
+        append_error = splt_t_append_splitpoint(state, temp2 * 100, NULL, SPLT_SPLITPOINT);
         if (append_error != SPLT_OK)
         {
           *error = append_error;
@@ -763,7 +768,7 @@ int splt_cddb_put_splitpoints (const char *file, splt_state *state, int *error)
       else
       {
         //we append 0 as splitpoint
-        append_error = splt_t_append_splitpoint(state, 0, NULL);
+        append_error = splt_t_append_splitpoint(state, 0, NULL, SPLT_SPLITPOINT);
         if (append_error != SPLT_OK)
         {
           *error = append_error;
@@ -925,13 +930,8 @@ int splt_cddb_put_splitpoints (const char *file, splt_state *state, int *error)
             }
 
             perfor = splt_t_get_tags_char_field(state, j-1, SPLT_TAGS_PERFORMER);
-            splt_u_cleanstring(state, perfor, error);
-            if (*error < 0) { goto function_end; }
             number = c+1;
           }
-
-          splt_u_cleanstring(state, number, error);
-          if (*error < 0) { goto function_end; }
         }
 
         //we limit number to 512?
@@ -1015,8 +1015,6 @@ int splt_cddb_put_splitpoints (const char *file, splt_state *state, int *error)
                   break;
                 }
               }
-              splt_u_cleanstring(state, artist, error);
-              if (*error < 0) { goto function_end; }
 
               //put artist info to client
               client_infos = malloc(sizeof(char) * (strlen(artist)+30));
@@ -1043,8 +1041,6 @@ int splt_cddb_put_splitpoints (const char *file, splt_state *state, int *error)
               }
 
               album = splt_t_get_tags_char_field(state,0, SPLT_TAGS_ALBUM);
-              splt_u_cleanstring(state, album, error);
-              if (*error < 0) { goto function_end; }
 
               //put album info to client
               client_infos = malloc(sizeof(char) * (strlen(album)+30));
@@ -1737,8 +1733,8 @@ char *splt_freedb_get_file(splt_state *state, int i, int *error,
   if (winsockinit != 0)
   {
     splt_t_clean_strerror_msg(state);
-    error = SPLT_FREEDB_ERROR_INITIALISE_SOCKET;
-    return error;
+    *error = SPLT_FREEDB_ERROR_INITIALISE_SOCKET;
+    return output;
   }
 #else
   int fd = 0;
@@ -2158,8 +2154,8 @@ end_function:
     if ((c=getenv("HOME"))!=NULL) sprintf(message, "%s/"PROXYCONFIG, c);
     else strncpy(message, PROXYCONFIG, strlen(PROXYCONFIG));
 
-    if (!(output=fopen(message, "r"))) {
-    if (!(output=fopen(message, "w+"))) {
+    if (!(output=splt_u_fopen(message, "r"))) {
+    if (!(output=splt_u_fopen(message, "w+"))) {
     fprintf(stderr, "\nWARNING Can't open config file ");
     perror(message);
     }
