@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2008 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2009 Alexandru Munteanu - io_fx@yahoo.fr
  *
  * http://mp3splt.sourceforge.net
  *
@@ -156,12 +156,9 @@ void splt_check_if_new_filename_path_correct(splt_state *state,
   if ((strcmp(new_filename_path, "") != 0) &&
       (strcmp(new_filename_path, current_directory) != 0))
   {
-    //used to see if the file exists
-    struct stat buffer;
-    int         status;
-
-    //-1 means error
-    if((status = stat(new_filename_path, &buffer)) == -1)
+    mode_t st_mode;
+    int status = splt_u_stat(new_filename_path, &st_mode, NULL);
+    if(status == -1)
     {
       splt_t_set_strerror_msg(state);
       splt_t_set_error_data(state, new_filename_path);
@@ -169,10 +166,8 @@ void splt_check_if_new_filename_path_correct(splt_state *state,
     }
     else
     {
-      //if it is a directory
-      if (S_ISDIR(buffer.st_mode) != 0)
+      if (S_ISDIR(st_mode) != 0)
       {
-        //no error
         return;
       }
       else
@@ -269,15 +264,19 @@ char *splt_check_put_dir_of_cur_song(const char *filename,
 
   if (new_filename_path != NULL)
   {
+#ifdef __WIN32__
     size_t path_len = strlen(new_filename_path);
     //erase the last char directory
     //-for windows manage c:\ because the gtk dir returns us "c:\"
     //and the normal directories without the "\"
-    //-for unix, erase '/'
-    if (new_filename_path[path_len-1] == SPLT_DIRCHAR)
+    if (path_len > 3)
     {
-      new_filename_path[path_len-1] = '\0';
+      if (new_filename_path[path_len-1] == SPLT_DIRCHAR)
+      {
+        new_filename_path[path_len-1] = '\0';
+      }
     }
+#endif
   }
 
   return new_filename_path;
@@ -400,7 +399,7 @@ void splt_check_file_type(splt_state *state, int *error)
     splt_u_print_debug("Verify if the file is a file",0,filename);
 
     FILE *test = NULL;
-    if ((test = fopen(filename,"r")) != NULL)
+    if ((test = splt_u_fopen(filename,"r")) != NULL)
     {
       if (fclose(test) != 0)
       {
@@ -444,9 +443,6 @@ void splt_check_file_type(splt_state *state, int *error)
 //-we are not interested in errors
 int splt_check_is_file(splt_state *state, const char *fname)
 {
-  struct stat buffer;
-  int status = 0;
-
   if (fname == NULL)
   {
     return SPLT_FALSE;
@@ -458,12 +454,12 @@ int splt_check_is_file(splt_state *state, const char *fname)
     {
       return SPLT_TRUE;
     }
-    //not stdin :
-    status = stat(fname, &buffer);
+
+    mode_t st_mode;
+    int status = splt_u_stat(fname, &st_mode, NULL);
     if (status == 0)
     {
-      //if it is a file
-      if (S_ISREG(buffer.st_mode))
+      if (S_ISREG(st_mode))
       {
         return SPLT_TRUE;
       }
@@ -537,7 +533,7 @@ int splt_check_is_the_same_file(splt_state *state, const char *file1,
   if (is_file1 && is_file2)
   {
     //file1
-    if ((file1_ = fopen(file1,"r")) == NULL)
+    if ((file1_ = splt_u_fopen(file1,"r")) == NULL)
     {
       goto end_error;
     }
@@ -548,7 +544,7 @@ int splt_check_is_the_same_file(splt_state *state, const char *file1,
       if (fstat(file1_d,&file1_stat) == 0)
       {
         //file2
-        if ((file2_ = fopen(file2,"r")) == NULL)
+        if ((file2_ = splt_u_fopen(file2,"r")) == NULL)
         {
           goto end_error;
         }
@@ -644,13 +640,11 @@ int splt_check_is_the_same_file(splt_state *state, const char *file1,
     return SPLT_FALSE;
   }
 
-end:
-  return SPLT_FALSE;
-
 end_error:
   /*splt_t_set_strerror_msg(state);
   splt_t_set_error_data(state,file1);
   *error = SPLT_ERROR_CANNOT_OPEN_FILE;*/
+
   close_files(state, file1, &file1_,file2, &file2_,error);
   return SPLT_FALSE;
 }
