@@ -280,7 +280,7 @@ int mp3splt_set_progress_function(splt_state *state,
 }
 
 int mp3splt_set_silence_level_function(splt_state *state,
-  void (*get_silence_cb)(float level, void *user_data),
+  void (*get_silence_cb)(long time, float level, void *user_data),
   void *data)
 {
   int error = SPLT_OK;
@@ -305,7 +305,7 @@ int mp3splt_set_silence_level_function(splt_state *state,
 //split_value is which splitpoint hundreths of seconds
 //if split_value is LONG_MAX, we put the end of the song (EOF)
 int mp3splt_append_splitpoint(splt_state *state,
-    long split_value, const char *name)
+    long split_value, const char *name, int type)
 {
   int error = SPLT_OK;
 
@@ -315,7 +315,7 @@ int mp3splt_append_splitpoint(splt_state *state,
     {
       splt_t_lock_library(state);
 
-      error = splt_t_append_splitpoint(state, split_value, name);
+      error = splt_t_append_splitpoint(state, split_value, name, type);
 
       splt_t_unlock_library(state);
     }
@@ -620,7 +620,7 @@ int mp3splt_split(splt_state *state)
         splt_t_set_int_option(state, SPLT_OPT_INPUT_NOT_SEEKABLE, SPLT_TRUE);
       }
 
-      state->cancel_split = SPLT_FALSE;
+      splt_t_set_stop_split(state, SPLT_FALSE);
 
       //we set default internal options
       splt_t_set_default_iopts(state);
@@ -634,8 +634,6 @@ int mp3splt_split(splt_state *state)
       //we put the real splitnumber in the splitnumber variable
       //that could be changed (see splitnumber in mp3splt.h)
       state->split.splitnumber = state->split.real_splitnumber;
-
-      //we put splitnumber in the state, we will use it later
       splt_t_set_current_split(state,0);
 
       //we check if the filename is a real file
@@ -679,11 +677,14 @@ int mp3splt_split(splt_state *state)
         //normal split checks
         if (split_type == SPLT_OPTION_NORMAL_MODE)
         {
-          //check if we have at least 2 splitpoints
-          if (splt_t_get_splitnumber(state) < 2)
+          if (! splt_t_get_int_option(state, SPLT_OPT_PRETEND_TO_SPLIT))
           {
-            error = SPLT_ERROR_SPLITPOINTS;
-            goto function_end;
+            //check if we have at least 2 splitpoints
+            if (splt_t_get_splitnumber(state) < 2)
+            {
+              error = SPLT_ERROR_SPLITPOINTS;
+              goto function_end;
+            }
           }
 
           //we check if the splitpoints are in order
@@ -695,8 +696,8 @@ int mp3splt_split(splt_state *state)
         splt_t_set_new_filename_path(state, new_filename_path, &error);
         if (error < 0) { goto function_end; }
 
-        //we create output directory if it does not exists
-        error = splt_u_create_directory(state, new_filename_path);
+        //we create output directories if they do not exist
+        error = splt_u_create_directories(state, new_filename_path);
         if (error < 0) { goto function_end; }
 
         //check means the test is ok
@@ -733,7 +734,7 @@ int mp3splt_split(splt_state *state)
         splt_t_get_original_tags(state,&error);
         if (error < 0)
         {
-          splt_p_end(state);
+          splt_p_end(state, &error);
           goto function_end;
         }
 
@@ -770,10 +771,11 @@ int mp3splt_split(splt_state *state)
           case SPLT_OPTION_TIME_MODE:
             splt_s_time_split(state, &error);
             break;
+          case SPLT_OPTION_ERROR_MODE:
+            splt_s_error_split(state, &error);
+            break;
           default:
-            //this is the normal split or error mode split:
-            //here we also have error_mode and frame_mode 
-
+            //this is the normal split
             if (split_type == SPLT_OPTION_NORMAL_MODE)
             {
               //if we don't have STDIN
@@ -791,7 +793,7 @@ int mp3splt_split(splt_state *state)
         }
 
         //ends the 'init' of the plugin for the split
-        splt_p_end(state);
+        splt_p_end(state, &error);
 
 function_end:
         //free memory
@@ -919,7 +921,8 @@ const splt_freedb_results *mp3splt_get_freedb_search(splt_state *state,
   if (state != NULL)
   {
     //we copy the search string, in order not to modify the original one
-    char *search = strdup(search_string);
+    char *search = strdup((char *)search_string);
+
     if (search != NULL)
     {
       //puts the results in "search_results"
@@ -983,7 +986,7 @@ void mp3splt_write_freedb_file_result(splt_state *state, int disc_id,
       {
         //we write the result to the file
         FILE *output = NULL;
-        if (!(output = fopen(cddb_file, "w")))
+        if (!(output = splt_u_fopen(cddb_file, "w")))
         {
           splt_t_set_strerror_msg(state);
           splt_t_set_error_data(state,cddb_file);
@@ -1080,7 +1083,7 @@ const splt_syncerrors *mp3splt_get_syncerrors(splt_state *state,
         {
           splt_t_unlock_messages(state);
           splt_p_search_syncerrors(state, err);
-          splt_p_end(state);
+          splt_p_end(state, err);
         }
         else
         {
@@ -1137,7 +1140,7 @@ const splt_wrap *mp3splt_get_wrap_files(splt_state *state,
         {
           splt_t_unlock_messages(state);
           splt_p_dewrap(state, SPLT_TRUE, NULL, err);
-          splt_p_end(state);
+          splt_p_end(state, err);
         }
         else
         {
@@ -1162,13 +1165,16 @@ const splt_wrap *mp3splt_get_wrap_files(splt_state *state,
   }
 }
 
-//count how many silence splitpoints we have with silence detection
-int mp3splt_count_silence_points(splt_state *state, int *error)
+//set the silence splitpoints without splitting
+int mp3splt_set_silence_points(splt_state *state, int *error)
 {
   int erro = SPLT_OK;
   int *err = &erro;
   if (error != NULL) { err = error; }
 
+  //set silence_mode option
+  mp3splt_set_int_option(state, SPLT_OPT_SPLIT_MODE, SPLT_OPTION_SILENCE_MODE);
+
   int found_splitpoints = -1;
 
   if (state != NULL)
@@ -1177,18 +1183,17 @@ int mp3splt_count_silence_points(splt_state *state, int *error)
     {
       splt_t_lock_library(state);
 
+      splt_t_set_stop_split(state, SPLT_FALSE);
+
       splt_check_file_type(state, err);
 
-      if (err >= 0)
+      if (*err >= 0)
       {
         splt_p_init(state, err);
-        if (err >= 0)
+        if (*err >= 0)
         {
-          found_splitpoints = splt_s_set_silence_splitpoints(state, SPLT_FALSE, err);
-          //the set_silence_splitpoints returns us the
-          //number of tracks, not splitpoints
-          found_splitpoints--;
-          splt_p_end(state);
+          found_splitpoints = splt_s_set_silence_splitpoints(state, err);
+          splt_p_end(state, err);
         }
       }
 
@@ -1207,6 +1212,14 @@ int mp3splt_count_silence_points(splt_state *state, int *error)
   return found_splitpoints;
 }
 
+//count how many silence splitpoints we have with silence detection
+int mp3splt_count_silence_points(splt_state *state, int *error)
+{
+  int number_of_tracks = mp3splt_set_silence_points(state, error) - 1;
+
+  return number_of_tracks;
+}
+
 //returns libmp3splt version, max 20 chars
 void mp3splt_get_version(char *version)
 {
@@ -1219,3 +1232,17 @@ char *mp3splt_get_strerror(splt_state *state, int error_code)
   return splt_u_strerror(state, error_code);
 }
 
+//returns possible error or SPLT_OK if no error
+int mp3splt_append_plugins_scan_dir(splt_state *state, char *dir)
+{
+  return splt_p_append_plugin_scan_dir(state, dir);
+}
+
+#ifdef __WIN32__
+//returned result must be free'd
+char *mp3splt_win32_utf16_to_utf8(const wchar_t *source)
+{
+  return splt_u_win32_utf16_to_utf8(source);
+}
+#endif
+
