@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2008 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2009 Alexandru Munteanu - io_fx@yahoo.fr
  *
  * http://mp3splt.sourceforge.net
  *
@@ -51,6 +51,43 @@ static void splt_t_free_oformat(splt_state *state);
 static void splt_t_state_put_default_options(splt_state *state, int *error);
 static void splt_t_free_files(char **files, int number);
 
+//returns SPLT_TRUE if the filename to split looks like STDIN
+int splt_t_is_stdin(splt_state *state)
+{
+  char *filename = splt_t_get_filename_to_split(state);
+
+  if (filename)
+  {
+    if ((strcmp(filename,"-") == 0) ||
+        (filename[strlen(filename)-1] == '-'))
+    {
+      return SPLT_TRUE;
+    }
+  }
+
+  return SPLT_FALSE;
+}
+
+//returns SPLT_TRUE if the output filename to split looks like STDOUT
+int splt_t_is_stdout(splt_state *state)
+{
+  const char *oformat = splt_t_get_oformat(state);
+
+  if (oformat)
+  {
+    if ((strcmp(oformat,"-") == 0))
+    {
+      return SPLT_TRUE;
+    }
+    else
+    {
+      return SPLT_FALSE;
+    }
+  }
+
+  return SPLT_FALSE;
+}
+
 /********************************/
 /* types: state access */
 
@@ -104,6 +141,13 @@ splt_state *splt_t_new_state(splt_state *state, int *error)
     splt_t_state_put_default_options(state, error);
   }
 
+  //set the default plugins scan directories
+  int return_value = splt_p_set_default_plugins_scan_dirs(state);
+  if (return_value != SPLT_OK)
+  {
+    return NULL;
+  }
+
   return state;
 }
 
@@ -198,7 +242,7 @@ int splt_t_alloc_init_new_plugin(splt_plugins *pl)
 }
 
 //frees the structure of one plugin data
-static void splt_t_free_plugin_data(splt_plugin_data pl_data)
+void splt_t_free_plugin_data(splt_plugin_data pl_data)
 {
   if (pl_data.info.name)
   {
@@ -526,6 +570,22 @@ char *splt_t_get_silence_log_fname(splt_state *state)
 /********************************/
 /* types: current split access */
 
+//functions for the current split file number
+static void splt_t_set_current_split_file_number(splt_state *state, int index)
+{
+  state->split.current_split_file_number = index;
+}
+
+static void splt_t_set_current_split_file_number_next(splt_state *state)
+{
+  splt_t_set_current_split_file_number(state, state->split.current_split_file_number+1);
+}
+
+int splt_t_get_current_split_file_number(splt_state *state)
+{
+  return state->split.current_split_file_number;
+}
+
 //sets the current split
 void splt_t_set_current_split(splt_state *state, int index)
 {
@@ -533,7 +593,26 @@ void splt_t_set_current_split(splt_state *state, int index)
   //(index >= 0))
   if (index >= 0)
   {
-    state->split.current_split = index;
+	  if (index == 0)
+	  {
+		  splt_t_set_current_split_file_number(state,1);
+	  }
+	  else
+	  {
+		  if (splt_t_splitpoint_exists(state, index))
+		  {
+			  int err = SPLT_OK;
+			  if (splt_t_get_splitpoint_type(state, index, &err) != SPLT_SKIPPOINT)
+			  {
+				  splt_t_set_current_split_file_number_next(state);
+			  }
+		  }
+		  else
+		  {
+			  splt_t_set_current_split_file_number_next(state);
+		  }
+	  }
+	  state->split.current_split = index;
   }
   else
   {
@@ -593,12 +672,17 @@ int splt_t_new_oformat(splt_state *state, const char *format_string)
   return error;
 }
 
+void splt_t_set_oformat_digits_tracks(splt_state *state, int tracks)
+{
+  int i = 0;
+  i = (int) (log10((double) (tracks)));
+  state->oformat.output_format_digits = (char) ((i+1) | 0x30);
+}
+
 //puts the number of digits for the output format
 void splt_t_set_oformat_digits(splt_state *state)
 {
-  int i = 0;
-  i = (int) (log10((double) (splt_t_get_splitnumber(state))));
-  state->oformat.output_format_digits = (char) ((i+1) | 0x30);
+  splt_t_set_oformat_digits_tracks(state, splt_t_get_splitnumber(state));
 }
 
 //puts the output format
@@ -635,6 +719,8 @@ void splt_t_set_oformat(splt_state *state, const char *format_string, int *error
   {
     splt_t_set_oformat_digits(state); 
   }
+
+  splt_t_set_int_option(state, SPLT_OPT_OUTPUT_FILENAMES, SPLT_OUTPUT_FORMAT);
 }
 
 //returns the output format string
@@ -674,7 +760,9 @@ int splt_t_get_splitnumber(splt_state *state)
 //puts a splitpoint in the state with an eventual file name
 //split_value is which splitpoint hundreths of seconds
 //if split_value is LONG_MAX, we put the end of the song (EOF)
-int splt_t_append_splitpoint(splt_state *state, long split_value, const char *name)
+//-the 'type' of the splitpoint can be : 
+//  SPLT_SPLITPOINT or SPLT_SKIPPOINT
+int splt_t_append_splitpoint(splt_state *state, long split_value, const char *name, int type)
 {
   int error = SPLT_OK;
 
@@ -725,6 +813,9 @@ int splt_t_append_splitpoint(splt_state *state, long split_value, const char *na
       error = name_error;
       return error;
     }
+
+    //set splitpoint type
+    splt_t_set_splitpoint_type(state, state->split.real_splitnumber - 1, type);
   }
   else
   {
@@ -836,6 +927,24 @@ int splt_t_set_splitpoint_name(splt_state *state, int index, const char *name)
   return error;
 }
 
+//sets the type of the splitpoint
+int splt_t_set_splitpoint_type(splt_state *state, int index, int type)
+{
+  int error = SPLT_OK;
+
+  if ((index >= 0) &&
+      (index < state->split.real_splitnumber))
+  {
+    state->split.points[index].type = type;
+  }
+  else
+  {
+    splt_u_error(SPLT_IERROR_INT,__func__, index, NULL);
+  }
+
+  return error;
+}
+
 //returns the splitpoints from the state
 splt_point *splt_t_get_splitpoints(splt_state *state, int *splitpoints_number)
 {
@@ -873,9 +982,53 @@ char *splt_t_get_splitpoint_name(splt_state *state, int index, int *error)
   }
 }
 
+//get the splitpoint type
+int splt_t_get_splitpoint_type(splt_state *state, int index, int *error)
+{
+  if ((index >= 0) &&
+      (index < state->split.real_splitnumber))
+  {
+    return state->split.points[index].type;
+  }
+  else
+  {
+    //splt_u_error(SPLT_IERROR_INT,__func__, index, NULL);
+    return 1;
+  }
+}
+
 /********************************/
 /* types: tags access */
 
+//if we auto increment the track number, we must replace the tracknumber in 
+//'all remaining tags like X'
+void splt_t_set_auto_increment_tracknumber_tag(splt_state *state, int old_current_split, int current_split)
+{
+  int remaining_tags_like_x = splt_t_get_int_option(state,SPLT_OPT_ALL_REMAINING_TAGS_LIKE_X); 
+
+  if (remaining_tags_like_x != -1)
+  {
+    if (splt_t_get_int_option(state, SPLT_OPT_AUTO_INCREMENT_TRACKNUMBER_TAGS) > 0)
+    {
+      if (current_split == remaining_tags_like_x)
+      {
+        if ((old_current_split > 0) && 
+            (old_current_split-1 < state->split.real_tagsnumber) && 
+            (old_current_split != remaining_tags_like_x))
+        {
+          int previous_track = splt_t_get_tags_int_field(state, old_current_split - 1, SPLT_TAGS_TRACK);
+          splt_t_set_tags_int_field(state, remaining_tags_like_x, SPLT_TAGS_TRACK, previous_track);
+        }
+        if (old_current_split != current_split)
+        {
+          int tracknumber = splt_t_get_tags_int_field(state, current_split, SPLT_TAGS_TRACK);
+          splt_t_set_tags_int_field(state, current_split, SPLT_TAGS_TRACK, tracknumber+1);
+        }
+      }
+    }
+  }
+}
+
 //sets the original tags to the state
 void splt_t_get_original_tags(splt_state *state, int *err)
 {
@@ -1002,7 +1155,7 @@ int splt_t_append_only_non_null_previous_tags(splt_state *state,
     }
     if (error != SPLT_OK)
       return error;
-    if (track != -1)
+    if (track != -INT_MAX)
     {
       error = splt_t_set_tags_int_field(state,
           old_tagsnumber, SPLT_TAGS_TRACK, track);
@@ -1029,8 +1182,9 @@ static void splt_t_set_empty_tags(splt_state *state, int index)
   state->split.tags[index].performer = NULL;
   state->split.tags[index].year = NULL;
   state->split.tags[index].comment = NULL;
-  state->split.tags[index].track = 0;
+  state->split.tags[index].track = -INT_MAX;
   state->split.tags[index].genre = 0x0;
+  state->split.tags[index].tags_version = 0;
 }
 
 //allocate new tags if necessary
@@ -1302,7 +1456,7 @@ void splt_t_clean_original_tags(splt_state *state)
     free(state->original_tags.comment);
     state->original_tags.comment = NULL;
   }
-  state->original_tags.track = -1;
+  state->original_tags.track = -INT_MAX;
   //12 means "other"
   state->original_tags.genre = 12;
 }
@@ -1467,6 +1621,9 @@ int splt_t_set_original_tags_field(splt_state *state,
     case SPLT_TAGS_GENRE:
       state->original_tags.genre = uchar_data;
       break;
+    case SPLT_TAGS_VERSION:
+      state->original_tags.tags_version = int_data;
+      break;
     default:
       splt_u_error(SPLT_IERROR_INT,__func__, -500, NULL);
       break;
@@ -1512,6 +1669,10 @@ int splt_t_set_tags_int_field(splt_state *state, int index,
         splt_u_print_debug("Setting track tags at",index,temp);
         state->split.tags[index].track = data;
         break;
+      case SPLT_TAGS_VERSION:
+        splt_u_print_debug("Setting tags version at",index,temp);
+        state->split.tags[index].tags_version = data;
+        break;
       default:
         break;
     }
@@ -1626,6 +1787,9 @@ int splt_t_get_tags_int_field(splt_state *state, int index, int tags_field)
       case SPLT_TAGS_TRACK:
         return state->split.tags[index].track;
         break;
+      case SPLT_TAGS_VERSION:
+        return state->split.tags[index].tags_version;
+        break;
       default:
         splt_u_error(SPLT_IERROR_INT,__func__, index, NULL);
         break;
@@ -1831,6 +1995,7 @@ static void splt_t_state_put_default_options(splt_state *state, int *error)
   state->split.real_tagsnumber = 0;
   state->split.real_splitnumber = 0;
   state->split.splitnumber = 0;
+  state->split.current_split_file_number = 1;
   state->split.get_silence_level = NULL;
   //plugins
   state->plug->plugins_scan_dirs = NULL;
@@ -1874,25 +2039,30 @@ static void splt_t_state_put_default_options(splt_state *state, int *error)
   state->iopts.split_begin = 0;
   state->iopts.split_end = 0;
   //options
-  state->options.output_filenames = SPLT_OUTPUT_DEFAULT;
   state->options.split_mode = SPLT_OPTION_NORMAL_MODE;
   state->options.tags = SPLT_CURRENT_TAGS;
+  state->options.output_filenames = SPLT_OUTPUT_DEFAULT;
+  state->options.quiet_mode = SPLT_FALSE;
+  state->options.pretend_to_split = SPLT_FALSE;
   state->options.option_frame_mode = SPLT_TRUE;
+  state->options.split_time = 6000;
   state->options.option_auto_adjust = SPLT_FALSE;
   state->options.option_input_not_seekable = SPLT_FALSE;
-  state->options.parameter_number_tracks = SPLT_DEFAULT_PARAM_TRACKS;
-  state->options.parameter_remove_silence = SPLT_FALSE;
-  state->options.parameter_gap = SPLT_DEFAULT_PARAM_GAP;
+  state->options.create_dirs_from_filenames = SPLT_FALSE;
   state->options.parameter_threshold = SPLT_DEFAULT_PARAM_THRESHOLD;
   state->options.parameter_offset = SPLT_DEFAULT_PARAM_OFFSET;
+  state->options.parameter_number_tracks = SPLT_DEFAULT_PARAM_TRACKS;
   state->options.parameter_minimum_length = SPLT_DEFAULT_PARAM_MINIMUM_LENGTH;
-  state->options.split_time = 6000;
+  state->options.parameter_remove_silence = SPLT_FALSE;
+  state->options.parameter_gap = SPLT_DEFAULT_PARAM_GAP;
   //error strings for error messages
   state->err.error_data = NULL;
   state->err.strerror_msg = NULL;
 
   state->options.remaining_tags_like_x = -1;
+  state->options.auto_increment_tracknumber_tags = 0;
   state->options.enable_silence_log = SPLT_FALSE;
+  state->options.force_tags_version = 0;
 }
 
 //sets the error data information
@@ -2017,6 +2187,9 @@ void splt_t_set_int_option(splt_state *state, int option_name, int value)
     case SPLT_OPT_QUIET_MODE:
       state->options.quiet_mode = value;
       break;
+    case SPLT_OPT_PRETEND_TO_SPLIT:
+      state->options.pretend_to_split = value;
+      break;
     case SPLT_OPT_OUTPUT_FILENAMES:
       state->options.output_filenames = value;
       break;
@@ -2026,6 +2199,9 @@ void splt_t_set_int_option(splt_state *state, int option_name, int value)
     case SPLT_OPT_TAGS:
       state->options.tags = value;
       break;
+    case SPLT_OPT_CREATE_DIRS_FROM_FILENAMES:
+      state->options.create_dirs_from_filenames = value;
+      break;
     case SPLT_OPT_FRAME_MODE:
       state->options.option_frame_mode = value;
       break;
@@ -2047,9 +2223,15 @@ void splt_t_set_int_option(splt_state *state, int option_name, int value)
     case SPLT_OPT_ALL_REMAINING_TAGS_LIKE_X:
       state->options.remaining_tags_like_x = value;
       break;
+    case SPLT_OPT_AUTO_INCREMENT_TRACKNUMBER_TAGS:
+      state->options.auto_increment_tracknumber_tags = value;
+      break;
     case SPLT_OPT_ENABLE_SILENCE_LOG:
       state->options.enable_silence_log = value;
       break;
+    case SPLT_OPT_FORCE_TAGS_VERSION:
+      state->options.force_tags_version = value;
+      break;
     default:
       splt_u_error(SPLT_IERROR_INT,__func__, option_name, NULL);
       break;
@@ -2087,6 +2269,9 @@ int splt_t_get_int_option(splt_state *state, int option_name)
     case SPLT_OPT_QUIET_MODE:
       return state->options.quiet_mode;
       break;
+    case SPLT_OPT_PRETEND_TO_SPLIT:
+      return state->options.pretend_to_split;
+      break;
     case SPLT_OPT_OUTPUT_FILENAMES:
       return state->options.output_filenames;
       break;
@@ -2096,6 +2281,9 @@ int splt_t_get_int_option(splt_state *state, int option_name)
     case SPLT_OPT_TAGS:
       return state->options.tags;
       break;
+    case SPLT_OPT_CREATE_DIRS_FROM_FILENAMES:
+      return state->options.create_dirs_from_filenames;
+      break;
     case SPLT_OPT_FRAME_MODE:
       return state->options.option_frame_mode;
       break;
@@ -2117,13 +2305,21 @@ int splt_t_get_int_option(splt_state *state, int option_name)
     case SPLT_OPT_ALL_REMAINING_TAGS_LIKE_X:
       return state->options.remaining_tags_like_x;
       break;
+    case SPLT_OPT_AUTO_INCREMENT_TRACKNUMBER_TAGS:
+      return state->options.auto_increment_tracknumber_tags;
+      break;
     case SPLT_OPT_ENABLE_SILENCE_LOG:
       return state->options.enable_silence_log;
       break;
+    case SPLT_OPT_FORCE_TAGS_VERSION:
+      return state->options.force_tags_version;
+      break;
     default:
       splt_u_error(SPLT_IERROR_INT,__func__, option_name, NULL);
       break;
   }
+
+  return -1;
 }
 
 //returns a float option (see mp3splt_types.h for int options)
@@ -2674,7 +2870,7 @@ int splt_t_put_split_file(splt_state *state, const char *filename)
     {
       //we open the m3u file
       FILE *file_input = NULL;
-      if ((file_input = fopen(new_m3u_file, "a+")) != NULL)
+      if ((file_input = splt_u_fopen(new_m3u_file, "a+")) != NULL)
       {
         //we don't care about the path of the split filename
         fprintf(file_input,"%s\n",splt_u_get_real_name(filename));
@@ -2699,6 +2895,8 @@ int splt_t_put_split_file(splt_state *state, const char *filename)
   {
     //splt_u_error(SPLT_IERROR_INT,__func__, -500, NULL);
   }
+
+  return error;
 }
 
 //puts in the state->progress_text the real text 
@@ -2721,7 +2919,7 @@ void splt_t_put_progress_text(splt_state *state, int type)
   {
     char *point_name = NULL;
     int curr_split = splt_t_get_current_split(state);
-    point_name = splt_t_get_splitpoint_name(state, curr_split-1,&err);
+    point_name = splt_t_get_splitpoint_name(state, curr_split,&err);
 
     if (point_name != NULL)
     {
@@ -2744,7 +2942,7 @@ void splt_t_put_progress_text(splt_state *state, int type)
 
     snprintf(state->split.p_bar->filename_shorted, 512,"%s", filename_shorted);
 
-    state->split.p_bar->current_split = splt_t_get_current_split(state);
+    state->split.p_bar->current_split = splt_t_get_current_split_file_number(state);
     state->split.p_bar->max_splits = state->split.splitnumber-1;
     state->split.p_bar->progress_type = type;
   }
@@ -2839,41 +3037,6 @@ void splt_t_clean_one_split_data(splt_state *state, int num)
   }
 }
 
-//returns SPLT_TRUE if the filename to split looks like STDIN
-int splt_t_is_stdin(splt_state *state)
-{
-  char *filename = splt_t_get_filename_to_split(state);
-
-  if (filename)
-  {
-    if ((strcmp(filename,"-") == 0) ||
-        (filename[strlen(filename)-1] == '-'))
-    {
-      return SPLT_TRUE;
-    }
-  }
-
-  return SPLT_FALSE;
-}
-
-//returns SPLT_TRUE if the output filename to split looks like STDOUT
-int splt_t_is_stdout(splt_state *state)
-{
-  const char *oformat = splt_t_get_oformat(state);
-
-  if (oformat)
-  {
-    if ((strcmp(oformat,"-") == 0))
-    {
-      return SPLT_TRUE;
-    }
-    else
-    {
-      return SPLT_FALSE;
-    }
-  }
-}
-
 //frees an array of strings
 static void splt_t_free_files(char **files, int number)
 {
