@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2008 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2009 Alexandru Munteanu - io_fx@yahoo.fr
  *
  * http://mp3splt.sourceforge.net
  *
@@ -36,6 +36,11 @@
 #include <ctype.h>
 #include <dirent.h>
 
+#ifdef __WIN32__
+#include <windows.h>
+#include <direct.h>
+#endif
+
 #include "splt.h"
 
 extern short global_debug;
@@ -82,7 +87,7 @@ long splt_u_convert_hundreths (const char *s)
 float splt_u_convert2dB(double input)
 {
   float level;
-  if (input<=0.0)
+  if (input <= 0.0)
   {
     level = -96.0;
   }
@@ -156,8 +161,38 @@ off_t splt_u_flength(splt_state *state, FILE *in, const char *filename, int *err
 /*****************************************************/
 /* utils manipulating strings (including filenames) */
 
+int splt_u_is_illegal_char(char c, int ignore_dirchar)
+{
+/*#ifdef __WIN32__
+  char windows_illegal_characters[] =
+  { '\\', '/', ':', '*', '?', '"', "<", ">", "|", '\r' };
+#elif defined(__MACOS__)
+  char mac_os_illegal_characters[] = { ':' };
+#elif defined(__MACOSX__)
+  char mac_osx_illegal_characters[] = { ':' };
+#else
+  char _nix_illegal_characters[] = { '/' };
+#endif*/
+
+  if ((ignore_dirchar) && (c == SPLT_DIRCHAR))
+  {
+    return SPLT_FALSE;
+  }
+
+  //for the sake of filename portability, we take the the windows illegal
+  //characters (will be changed upon feature request)
+  if ((c == '\\') || (c == '/') || (c == ':') || (c == '*') ||
+      (c == '?') || (c == '"') || (c == '<') ||
+      (c == '>') || (c == '|') || (c == '\r'))
+  {
+    return SPLT_TRUE;
+  }
+
+  return SPLT_FALSE;
+}
+
 //cleans the string of weird characters like ? " | : > < * \ \r
-void splt_u_cleanstring(splt_state *state, char *s, int *error)
+void splt_u_cleanstring_(splt_state *state, char *s, int *error, int ignore_dirchar)
 {
   int i = 0, j=0;
   char *copy = NULL;
@@ -168,13 +203,14 @@ void splt_u_cleanstring(splt_state *state, char *s, int *error)
     {
       for (i=0; i<=strlen(copy); i++)
       {
-        if ((copy[i]!='\\')&&(copy[i]!='/')&&(copy[i]!='?')
-            &&(copy[i]!='*')&&(copy[i]!=':')&&(copy[i]!='"')
-            &&(copy[i]!='>')&&(copy[i]!='<')&&(copy[i]!='|')
-            &&(copy[i]!='\r'))
+        if (! splt_u_is_illegal_char(copy[i], ignore_dirchar))
         {
           s[j++] = copy[i];
         }
+        else
+        {
+          s[j++] = '_';
+        }
       }
       free(copy);
       copy = NULL;
@@ -199,6 +235,11 @@ void splt_u_cleanstring(splt_state *state, char *s, int *error)
   }
 }
 
+void splt_u_cleanstring(splt_state *state, char *s, int *error)
+{
+  splt_u_cleanstring_(state, s, error, SPLT_FALSE);
+}
+
 //cuts spaces from the begin
 char *splt_u_cut_spaces_from_begin(char *c)
 {
@@ -716,7 +757,11 @@ int splt_u_put_tags_from_string(splt_state *state, const char *tags, int *error)
     char *all_year = NULL;
     char *all_comment = NULL;
     int all_tracknumber = -1;
+    int auto_incremented_tracknumber = -1;
     unsigned char all_genre = 12;
+    //when using 'N', auto increment the track number
+    short auto_increment_tracknumber = SPLT_FALSE;
+    short first_time_auto_increment_tracknumber = SPLT_TRUE;
 
     int tags_appended = 0;
     int get_out_from_while = SPLT_FALSE;
@@ -813,7 +858,6 @@ int splt_u_put_tags_from_string(splt_state *state, const char *tags, int *error)
                 ambigous = SPLT_TRUE;
               }
 
-              char *filename = splt_t_get_filename_to_split(state);
               //if we don't have STDIN
               if (! splt_t_is_stdin(state))
               {
@@ -841,7 +885,7 @@ int splt_u_put_tags_from_string(splt_state *state, const char *tags, int *error)
                     get_out_from_while = SPLT_TRUE;
                     goto end_while;
                   }
-                  splt_p_end(state);
+                  splt_p_end(state, &err);
                   err = splt_t_append_original_tags(state);
                   if (err < 0)
                   {
@@ -1038,7 +1082,6 @@ int splt_u_put_tags_from_string(splt_state *state, const char *tags, int *error)
               if (*error < 0) { get_out_from_while = SPLT_TRUE; goto end_while; }
               if (tracknumber != NULL)
               {
-
                 cur_pos += strlen(tracknumber)+2;
                 s_tracknumber++;
               }
@@ -1046,6 +1089,24 @@ int splt_u_put_tags_from_string(splt_state *state, const char *tags, int *error)
               {
                 cur_pos++;
               }
+              if (auto_increment_tracknumber)
+              {
+                first_time_auto_increment_tracknumber = SPLT_TRUE;
+              }
+              break;
+            case 'N':
+              tracknumber = splt_u_parse_tag_word(cur_pos,end_paranthesis, &ambigous,error);
+              if (*error < 0) { get_out_from_while = SPLT_TRUE; goto end_while; }
+              splt_t_set_int_option(state,SPLT_OPT_AUTO_INCREMENT_TRACKNUMBER_TAGS, SPLT_TRUE);
+              if (auto_increment_tracknumber)
+              {
+                first_time_auto_increment_tracknumber = SPLT_TRUE;
+              }
+              else if (all_tags)
+              {
+                auto_increment_tracknumber = SPLT_TRUE;
+                first_time_auto_increment_tracknumber = SPLT_TRUE;
+              }
               break;
             default:
               ambigous = SPLT_TRUE;
@@ -1078,13 +1139,24 @@ int splt_u_put_tags_from_string(splt_state *state, const char *tags, int *error)
           track = atoi(tracknumber);
 
           //copy the tracknumber for all tags
-          if (all_tags)
+          if (all_tags || auto_increment_tracknumber)
           {
             all_tracknumber = track;
           }
         }
       }
 
+      //
+      if (auto_increment_tracknumber)
+      {
+        if (first_time_auto_increment_tracknumber)
+        {
+          auto_incremented_tracknumber = all_tracknumber;
+          first_time_auto_increment_tracknumber = SPLT_FALSE;
+        }
+        track = auto_incremented_tracknumber++;
+      }
+
       if ((s_title > 1) || (s_artist > 1)
           || (s_album > 1) || (s_performer > 1)
           || (s_year > 1) || (s_comment > 1)
@@ -1101,6 +1173,7 @@ int splt_u_put_tags_from_string(splt_state *state, const char *tags, int *error)
         {
           track = 0;
         }
+
         //we put the tags
         int err = splt_t_append_tags(state, title, artist,
             album, performer, year, comment, track, genre);
@@ -1150,7 +1223,7 @@ int splt_u_put_tags_from_string(splt_state *state, const char *tags, int *error)
         {
           splt_t_set_tags_char_field(state, index, SPLT_TAGS_COMMENT, all_comment);
         }
-        if (!tracknumber)
+        if (!tracknumber && ! auto_incremented_tracknumber)
         {
           splt_t_set_tags_int_field(state, index, SPLT_TAGS_TRACK, all_tracknumber);
         }
@@ -1225,9 +1298,6 @@ int splt_u_parse_outformat(char *s, splt_state *state)
     }
   }
 
-  splt_u_cleanstring(state, s, &amb);
-  if (amb < 0) { return amb; }
-
   ptrs = s;
   i = 0;
   ptre = strchr(ptrs+1, '%');
@@ -1329,8 +1399,7 @@ int splt_u_put_output_format_filename(splt_state *state)
   char *performer = NULL;
   char *original_filename = NULL;
 
-  int old_current_split = splt_t_get_current_split(state);
-
+  int old_current_split = splt_t_get_current_split_file_number(state) - 1;
   int current_split = old_current_split;
 
   int fm_length = 0;
@@ -1380,6 +1449,8 @@ int splt_u_put_output_format_filename(splt_state *state)
             //we get the artist
             artist =
               splt_t_get_tags_char_field(state,current_split, SPLT_TAGS_ARTIST);
+            splt_u_cleanstring(state, artist, &error);
+            if (error < 0) { goto end; };
           }
           else
           {
@@ -1423,6 +1494,8 @@ int splt_u_put_output_format_filename(splt_state *state)
             //we get the album
             album =
               splt_t_get_tags_char_field(state,current_split, SPLT_TAGS_ALBUM);
+            splt_u_cleanstring(state, album, &error);
+            if (error < 0) { goto end; };
           }
           else
           {
@@ -1464,8 +1537,9 @@ int splt_u_put_output_format_filename(splt_state *state)
           if (splt_t_tags_exists(state,current_split))
           {
             //we get the title
-            title =
-              splt_t_get_tags_char_field(state,current_split, SPLT_TAGS_TITLE);
+            title = splt_t_get_tags_char_field(state,current_split, SPLT_TAGS_TITLE);
+            splt_u_cleanstring(state, title, &error);
+            if (error < 0) { goto end; };
           }
           else
           {
@@ -1507,8 +1581,9 @@ int splt_u_put_output_format_filename(splt_state *state)
           if (splt_t_tags_exists(state,current_split))
           {
             //we get the performer
-            performer =
-              splt_t_get_tags_char_field(state,current_split, SPLT_TAGS_PERFORMER);
+            performer = splt_t_get_tags_char_field(state,current_split, SPLT_TAGS_PERFORMER);
+            splt_u_cleanstring(state, performer, &error);
+            if (error < 0) { goto end; };
           }
           else
           {
@@ -1550,8 +1625,24 @@ int splt_u_put_output_format_filename(splt_state *state)
           temp[2] = state->oformat.output_format_digits;
           temp[3] = 'd';
 
+          int number_of_digits = state->oformat.output_format_digits;
+
+          int format_length = strlen(state->oformat.format[i]);
+          char *format_dup = strdup(state->oformat.format[i]);
+          if ((format_length > 2) &&
+              isdigit(state->oformat.format[i][2]))
+          {
+            temp[2] = state->oformat.format[i][2];
+            number_of_digits = (int) state->oformat.format[i][2];
+            int k = 0;
+            for (k = 2;k < format_length;k++)
+            {
+              format_dup[k] = format_dup[k+1];
+            }
+          }
+
           //we set the track number
-          int tracknumber = old_current_split+1;
+          int tracknumber = old_current_split + 1;
 
           //if not time split, or normal split, or silence split or error,
           //we put the track number from the tags
@@ -1571,11 +1662,14 @@ int splt_u_put_output_format_filename(splt_state *state)
               }
             }
           }
-          snprintf(temp+4, temp_len, state->oformat.format[i]+2);
+          snprintf(temp+4, temp_len, format_dup+2);
+          if (format_dup)
+          {
+            free(format_dup);
+            format_dup = NULL;
+          }
 
-          fm_length = strlen(temp) + 1;
-          char tt[2] = { state->oformat.output_format_digits, '\0' };
-          int number_of_digits = atoi(tt);
+          fm_length = strlen(temp) + 1 + number_of_digits;
           if ((fm = malloc(fm_length * sizeof(char))) == NULL)
           {
             error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
@@ -1677,13 +1771,100 @@ int splt_u_put_output_format_filename(splt_state *state)
 
   splt_u_print_debug("The new output filename is ",0,output_filename);
 
-  name_error = splt_t_set_splitpoint_name(state, cur_splt, output_filename);
-
-  if (name_error != SPLT_OK)
+  if (splt_t_get_int_option(state, SPLT_OPT_CREATE_DIRS_FROM_FILENAMES))
   {
-    error = name_error;
+    char *only_dirs = strdup(output_filename);
+    if (! only_dirs)
+    {
+      error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+      goto end;
+    }
+
+    //get out the directory path
+    char *dir_char = strrchr(only_dirs, SPLT_DIRCHAR);
+    if (dir_char != NULL)
+    {
+      *dir_char = '\0';
+      char *path_of_split = splt_t_get_path_of_split(state);
+      //append the 'directory path' from the filename to the path of split
+      if (path_of_split)
+      {
+        int malloc_length = strlen(path_of_split) + strlen(only_dirs) + 3;
+        char *full_path_to_create = malloc(sizeof(char) * malloc_length);
+        if (! full_path_to_create)
+        {
+          error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+          if (only_dirs)
+          {
+            free(only_dirs);
+            only_dirs = NULL;
+          }
+          goto end;
+        }
+        else
+        {
+          snprintf(full_path_to_create, malloc_length - 1,
+              "%s%c%s", path_of_split, SPLT_DIRCHAR, only_dirs);
+          splt_u_create_directories(state, full_path_to_create);
+        }
+
+        if (full_path_to_create)
+        {
+          free(full_path_to_create);
+          full_path_to_create = NULL;
+        }
+      }
+      //the new path of split is the path of the source file
+      else
+      {
+        char *new_filename_path = splt_t_get_new_filename_path(state);
+        //append the new path of split from the filename to the path of split
+        if (new_filename_path)
+        {
+          int malloc_length = strlen(new_filename_path) + strlen(only_dirs) + 3;
+          char *full_path_to_create = malloc(sizeof(char) * malloc_length);
+          if (! full_path_to_create)
+          {
+            error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+            if (only_dirs)
+            {
+              free(only_dirs);
+              only_dirs = NULL;
+            }
+            goto end;
+          }
+          else
+          {
+            if (new_filename_path != NULL && new_filename_path[0] != '\0')
+            {
+              snprintf(full_path_to_create, malloc_length - 1,
+                  "%s%c%s", new_filename_path , SPLT_DIRCHAR, only_dirs);
+              splt_u_create_directories(state, full_path_to_create);
+            }
+            else
+            {
+              splt_u_create_directories(state, only_dirs);
+            }
+          }
+
+          if (full_path_to_create)
+          {
+            free(full_path_to_create);
+            full_path_to_create = NULL;
+          }
+        }
+      }
+    }
+    if (only_dirs)
+    {
+      free(only_dirs);
+      only_dirs = NULL;
+    }
   }
 
+  name_error = splt_t_set_splitpoint_name(state, cur_splt, output_filename);
+  if (name_error != SPLT_OK) { error = name_error; }
+
 end:
   //free memory
   if (output_filename)
@@ -1746,8 +1927,8 @@ void splt_u_error(int error_type, const char *function,
 //get silence position depending of the offset
 float splt_u_silence_position(struct splt_ssplit *temp, float off)
 {
-  float position = (temp->end_position - temp->begin_position);
-  position = temp->begin_position + (position*off);
+  float length_of_silence = (temp->end_position - temp->begin_position);
+  float position = temp->begin_position + (length_of_silence * off);
 
   return position;
 }
@@ -1965,12 +2146,12 @@ char *splt_u_strerror(splt_state *state, int error_code)
         break;
       case SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE:
         snprintf(error_msg,max_error_size,
-            " error: cannot write to output file '%s' : %s",
-            state->err.error_data,state->err.strerror_msg);
+            " error: cannot write to output file '%s'",
+            state->err.error_data);
         break;
       case SPLT_ERROR_WHILE_READING_FILE:
-        snprintf(error_msg,max_error_size, " error: error while reading file '%s'",
-            state->err.error_data,state->err.strerror_msg);
+        snprintf(error_msg,max_error_size, " error: error while reading file '%s' : %s",
+            state->err.error_data, state->err.strerror_msg);
         break;
       case SPLT_ERROR_SEEKING_FILE:
         snprintf(error_msg,max_error_size, " error: cannot seek file '%s'",
@@ -2124,7 +2305,7 @@ int splt_u_parse_ssplit_file(splt_state *state, FILE *log_file, int *error)
 }
 
 //create recursive directories
-int splt_u_create_directory(splt_state *state, const char *dir)
+int splt_u_create_directories(splt_state *state, const char *dir)
 {
   int result = SPLT_OK;
   const char *ptr = NULL;
@@ -2137,62 +2318,78 @@ int splt_u_create_directory(splt_state *state, const char *dir)
   {
     return SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
   }
-  DIR *d = NULL;
   
   splt_u_print_debug("Creating directory ...",0,dir);
   
   ptr = dir;
+#ifdef __WIN32__
+  int first_time = SPLT_TRUE;
+#endif
   while ((ptr = strchr(ptr, SPLT_DIRCHAR))!=NULL)
+  {
+		//handle C:DIRCHAR on windows
+#ifdef __WIN32__
+    if (first_time && (strlen(dir) > 2)
+        && (dir[1] == ':') && (dir[2] == SPLT_DIRCHAR))
     {
       ptr++;
-      strncpy(junk, dir, ptr-dir);
-      junk[ptr-dir] = '\0';
-      
-      splt_u_print_debug("directory ...",0, junk);
-      
-      if (!(d = opendir(junk)))
-        {
-#ifdef __WIN32__
-          if ((mkdir(junk))==-1)
-            {
-#else
-          if ((mkdir(junk, 0755))==-1)
-            {
-#endif
-              splt_t_set_strerror_msg(state);
-              splt_t_set_error_data(state,junk);
-              result = SPLT_ERROR_CANNOT_CREATE_DIRECTORY;
-              break;
-            }
-        }
-      else
-        {
-          closedir(d);
-        }
     }
+    first_time = SPLT_FALSE;
+#endif
+    strncpy(junk, dir, ptr-dir);
+    junk[ptr-dir] = '\0';
+    ptr++;
+
+    if (junk[0] != '\0')
+    {
+      if (! splt_u_check_if_directory(junk))
+      {
+        splt_u_print_debug("directory ...",0, junk);
+
+        if (result < 0) { goto end; }
+
+        //don't create output directories if we pretend to split
+        if (! splt_t_get_int_option(state, SPLT_OPT_PRETEND_TO_SPLIT))
+        {
+          if ((splt_u_mkdir(junk)) == -1)
+          {
+            splt_t_set_strerror_msg(state);
+            splt_t_set_error_data(state,junk);
+            result = SPLT_ERROR_CANNOT_CREATE_DIRECTORY;
+            goto end;
+          }
+        }
+      }
+    }
+  }
 
   //we have created all the directories except the last one
-  if (!(d = opendir(dir)))
-    {
-      splt_u_print_debug("final directory ...",0, dir);
+  char *last_dir = strdup(dir);
+  if (!last_dir)
+  {
+    result = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
+    goto end;
+  }
 
-#ifdef __WIN32__
-      if ((mkdir(dir))==-1)
-        {
- #else
-      if ((mkdir(dir, 0755))==-1)
-        {
-#endif
-          splt_t_set_strerror_msg(state);
-          splt_t_set_error_data(state,dir);
-          result = SPLT_ERROR_CANNOT_CREATE_DIRECTORY;
-        }
-    }
-  else
+  if (! splt_u_check_if_directory(last_dir))
+  {
+    splt_u_print_debug("final directory ...",0, last_dir);
+
+    if ((splt_u_mkdir(last_dir)) == -1)
     {
-      closedir(d);
+      splt_t_set_strerror_msg(state);
+      splt_t_set_error_data(state,last_dir);
+      result = SPLT_ERROR_CANNOT_CREATE_DIRECTORY;
     }
+  }
   
+  if (last_dir)
+  {
+    free(last_dir);
+    last_dir = NULL;
+  }
+
+end:
   if (junk)
   {
     free(junk);
@@ -2211,20 +2408,18 @@ void splt_u_print(char *mess)
 //check if its a directory
 int splt_u_check_if_directory(char *fname)
 {
-  struct stat buffer;
-  int         status = 0;
-
   if (fname == NULL)
   {
     return SPLT_FALSE;
   }
   else
   {
-    status = stat(fname, &buffer);
+    mode_t st_mode;
+    int status = splt_u_stat(fname, &st_mode, NULL);
     if (status == 0)
     {
       //if it is a directory
-      if (S_ISDIR(buffer.st_mode))
+      if (S_ISDIR(st_mode))
       {
         return SPLT_TRUE;
       }
@@ -2242,3 +2437,185 @@ int splt_u_check_if_directory(char *fname)
   return SPLT_FALSE;
 }
 
+#ifdef __WIN32__
+static wchar_t *splt_u_win32_encoding_to_utf16(UINT encoding, const char *source)
+{
+  wchar_t *dest = NULL;
+
+  int converted_size = MultiByteToWideChar(encoding, 0, source, -1, NULL, 0);
+  if (converted_size > 0)
+  {
+    dest = malloc(sizeof(wchar_t) * converted_size);
+    if (dest)
+    {
+      MultiByteToWideChar(encoding, 0, source, -1, dest, converted_size);
+    }
+  }
+
+  return dest;
+}
+
+static char *splt_u_win32_utf16_to_encoding(UINT encoding, const wchar_t *source)
+{
+  char *dest = NULL;
+
+  int converted_size = WideCharToMultiByte(encoding, 0, source, -1, NULL, 0, NULL, NULL);
+  if (converted_size > 0)
+  {
+    dest = malloc(sizeof(char *) * converted_size);
+    if (dest)
+    {
+      WideCharToMultiByte(encoding, 0, source, -1, dest, converted_size, NULL, NULL);
+    }
+  }
+
+  return dest;
+}
+
+wchar_t *splt_u_win32_utf8_to_utf16(const char *source)
+{
+  return splt_u_win32_encoding_to_utf16(CP_UTF8, source);
+}
+
+char *splt_u_win32_utf16_to_utf8(const wchar_t *source)
+{
+  return splt_u_win32_utf16_to_encoding(CP_UTF8, source);
+}
+
+static int splt_u_win32_check_if_encoding_is_utf8(const char *source)
+{
+  int is_utf8 = SPLT_FALSE;
+
+  if (source)
+  {
+    wchar_t *source_wchar = splt_u_win32_utf8_to_utf16(source);
+    if (source_wchar)
+    {
+      char *source2 = splt_u_win32_utf16_to_utf8(source_wchar);
+      if (source2)
+      {
+        if (strcmp(source, source2) == 0)
+        {
+          is_utf8 = SPLT_TRUE;
+        }
+
+        free(source2);
+        source2 = NULL;
+      }
+
+      free(source_wchar);
+      source_wchar = NULL;
+    }
+  }
+
+  return is_utf8;
+}
+#endif
+
+//windows filenames have to be converted to utf16
+FILE *splt_u_fopen(const char *filename, const char *mode)
+{
+#ifdef __WIN32__
+  if (splt_u_win32_check_if_encoding_is_utf8(filename))
+  {
+    wchar_t *wfilename = splt_u_win32_utf8_to_utf16(filename);
+    wchar_t *wmode = splt_u_win32_utf8_to_utf16(mode);
+
+    FILE *file = _wfopen(wfilename, wmode);
+
+    if (wfilename)
+    {
+      free(wfilename);
+      wfilename = NULL;
+    }
+
+    if (wmode)
+    {
+      free(wmode);
+      wmode = NULL;
+    }
+
+    return file;
+  }
+  else
+#endif
+  {
+    return fopen(filename, mode);
+  }
+}
+
+int splt_u_mkdir(const char *path)
+{
+#ifdef __WIN32__
+  if (splt_u_win32_check_if_encoding_is_utf8(path))
+  {
+    wchar_t *wpath = splt_u_win32_utf8_to_utf16(path);
+
+    int ret = _wmkdir(wpath);
+
+    if (wpath)
+    {
+      free(wpath);
+      wpath = NULL;
+    }
+
+    return ret;
+  }
+  else
+  {
+    return mkdir(path);
+  }
+#else
+  return mkdir(path, 0755);
+#endif
+}
+
+int splt_u_stat(const char *path, mode_t *st_mode, off_t *st_size)
+{
+#ifdef __WIN32__
+  if (splt_u_win32_check_if_encoding_is_utf8(path))
+  {
+    struct _stat buf;
+    wchar_t *wpath = splt_u_win32_utf8_to_utf16(path);
+
+    int ret = _wstat(wpath, &buf);
+
+    if (wpath)
+    {
+      free(wpath);
+      wpath = NULL;
+    }
+
+    if (st_mode != NULL)
+    {
+      *st_mode = buf.st_mode;
+    }
+
+    if (st_size != NULL)
+    {
+      *st_size = buf.st_size;
+    }
+
+    return ret;
+  }
+  else
+#endif
+  {
+    struct stat buf;
+
+    int ret = stat(path, &buf);
+
+    if (st_mode != NULL)
+    {
+      *st_mode = buf.st_mode;
+    }
+
+    if (st_size != NULL)
+    {
+      *st_size = buf.st_size;
+    }
+
+    return ret;
+  }
+}
+
